@@ -36,7 +36,9 @@
  */
 
 #define GLFS_LG_BASE            GLFS_MSGID_COMP_LIBGLUSTERFS
-#define GLFS_LG_NUM_MESSAGES    205
+
+#define GLFS_LG_NUM_MESSAGES    209
+
 #define GLFS_LG_MSGID_END       (GLFS_LG_BASE + GLFS_LG_NUM_MESSAGES + 1)
 /* Messaged with message IDs */
 #define glfs_msg_start_lg GLFS_LG_BASE, "Invalid: Start of messages"
@@ -1748,6 +1750,41 @@
  */
 #define LG_MSG_LEASEID_BUF_INIT_FAILED                   (GLFS_LG_BASE + 205)
 
+/*!
+ * @messageid
+ * @diagnosis
+ * @recommendedaction
+ *
+ */
+#define LG_MSG_PTHREAD_ATTR_INIT_FAILED                  (GLFS_LG_BASE + 206)
+
+/*!
+ * @messageid
+ * @diagnosis
+ * @recommendedaction
+ *
+ */
+
+#define LG_MSG_INVALID_INODE_LIST                         (GLFS_LG_BASE + 207)
+
+/*!
+ * @messageid
+ * @diagnosis
+ * @recommendedaction
+ *
+ */
+
+#define LG_MSG_COMPACT_FAILED                            (GLFS_LG_BASE + 208)
+
+/*!
+ * @messageid
+ * @diagnosis
+ * @recommendedaction
+ *
+ */
+
+#define LG_MSG_COMPACT_STATUS                            (GLFS_LG_BASE + 209)
+
 /*!
  * @messageid
  * @diagnosis
