@@ -35,12 +35,12 @@ from master import gmaster_builder
 import syncdutils
 from syncdutils import GsyncdError, select, privileged, boolify, funcode
 from syncdutils import umask, entry2pb, gauxpfx, errno_wrap, lstat
-from syncdutils import NoPurgeTimeAvailable, PartialHistoryAvailable
+from syncdutils import NoStimeAvailable, PartialHistoryAvailable
 from syncdutils import ChangelogException, ChangelogHistoryNotAvailable
 from syncdutils import get_changelog_log_level
 from syncdutils import CHANGELOG_AGENT_CLIENT_VERSION
 from gsyncdstatus import GeorepStatus
-
+from syncdutils import get_master_and_slave_data_from_args
 
 UrlRX = re.compile('\A(\w+)://([^ *?[]*)\Z')
 HostRX = re.compile('[a-zA-Z\d](?:[a-zA-Z\d.-]*[a-zA-Z\d])?', re.I)
@@ -529,6 +529,29 @@ class Server(object):
             else:
                 raise
 
+    @classmethod
+    @_pathguard
+    def entry_stime(cls, path, uuid):
+        """
+        entry_stime xattr to reduce the number of retry of Entry changes when
+        Geo-rep worker crashes and restarts. entry_stime is updated after
+        processing every changelog file. On failure and restart, worker only
+        have to reprocess the last changelog for Entry ops.
+        Xattr Key: <PFX>.<MASTERVOL_UUID>.<SLAVEVOL_UUID>.entry_stime
+        """
+        try:
+            val = Xattr.lgetxattr(path,
+                                  '.'.join([cls.GX_NSPACE, uuid,
+                                            'entry_stime']),
+                                  8)
+            return struct.unpack('!II', val)
+        except OSError:
+            ex = sys.exc_info()[1]
+            if ex.errno in (ENOENT, ENODATA, ENOTDIR):
+                return ex.errno
+            else:
+                raise
+
     @classmethod
     def node_uuid(cls, path='.'):
         try:
@@ -549,6 +572,17 @@ class Server(object):
                    [ENOENT],
                    [ESTALE, EINVAL])
 
+    @classmethod
+    @_pathguard
+    def set_entry_stime(cls, path, uuid, mark):
+        """set @mark as stime for @uuid on @path"""
+        errno_wrap(Xattr.lsetxattr,
+                   [path,
+                    '.'.join([cls.GX_NSPACE, uuid, 'entry_stime']),
+                    struct.pack('!II', *mark)],
+                   [ENOENT],
+                   [ESTALE, EINVAL])
+
     @classmethod
     @_pathguard
     def set_xtime(cls, path, uuid, mark):
@@ -630,16 +664,19 @@ class Server(object):
             # We do this for failing fops on Slave
             # Master should be logging this
             if cmd_ret is None:
-                return
+                return False
 
             if cmd_ret == EEXIST:
                 disk_gfid = cls.gfid_mnt(e['entry'])
-                if isinstance(disk_gfid, basestring):
-                    if e['gfid'] != disk_gfid:
-                        failures.append((e, cmd_ret, disk_gfid))
+                if isinstance(disk_gfid, basestring) and e['gfid'] != disk_gfid:
+                    failures.append((e, cmd_ret, disk_gfid))
+                else:
+                    return False
             else:
                 failures.append((e, cmd_ret))
 
+            return True
+
         failures = []
 
         def matching_disk_gfid(gfid, entry):
@@ -784,11 +821,13 @@ class Server(object):
                         if st.st_ino == st1.st_ino:
                             # we have a hard link, we can now unlink source
                             try:
-                                os.unlink(entry)
+                                errno_wrap(os.unlink, [entry],
+                                           [ENOENT, ESTALE])
                             except OSError as e:
                                 if e.errno == EISDIR:
                                     try:
-                                        os.rmdir(entry)
+                                        errno_wrap(os.rmdir, [entry],
+                                                   [ENOENT, ESTALE])
                                     except OSError as e:
                                         if e.errno == ENOTEMPTY:
                                             logging.error(
@@ -807,7 +846,15 @@ class Server(object):
                                      [pg, 'glusterfs.gfid.newfile', blob],
                                      [EEXIST, ENOENT],
                                      [ESTALE, EINVAL])
-                collect_failure(e, cmd_ret)
+                failed = collect_failure(e, cmd_ret)
+
+                # If directory creation is failed, return immediately before
+                # further processing. Allowing it to further process will
+                # cause the entire directory tree to fail syncing to slave.
+                # Hence master will log and raise exception if it's
+                # directory failure.
+                if failed and op == 'MKDIR':
+                    return failures
 
                 # If UID/GID is different than zero that means we are trying
                 # create Entry with different UID/GID. Create Entry with
@@ -816,7 +863,7 @@ class Server(object):
                     path = os.path.join(pfx, gfid)
                     cmd_ret = errno_wrap(os.chown, [path, uid, gid], [ENOENT],
                                          [ESTALE, EINVAL])
-                collect_failure(e, cmd_ret)
+                    collect_failure(e, cmd_ret)
 
         return failures
 
@@ -959,6 +1006,12 @@ class SlaveRemote(object):
         so = getattr(gconf, 'session_owner', None)
         if so:
             extra_opts += ['--session-owner', so]
+        li = getattr(gconf, 'local_id', None)
+        if li:
+            extra_opts += ['--local-id', li]
+        ln = getattr(gconf, 'local_node', None)
+        if ln:
+            extra_opts += ['--local-node', ln]
         if boolify(gconf.use_rsync_xattrs):
             extra_opts.append('--use-rsync-xattrs')
         po = Popen(rargs + gconf.remote_gsyncd.split() + extra_opts +
@@ -1001,7 +1054,10 @@ class SlaveRemote(object):
             (boolify(gconf.sync_acls) and ['--acls'] or []) + \
             ['.'] + list(args)
 
-        if gconf.log_rsync_performance:
+        log_rsync_performance = boolify(gconf.configinterface.get_realtime(
+            "log_rsync_performance", default_value=False))
+
+        if log_rsync_performance:
             # use stdout=PIPE only when log_rsync_performance enabled
             # Else rsync will write to stdout and nobody is their
             # to consume. If PIPE is full rsync hangs.
@@ -1020,7 +1076,7 @@ class SlaveRemote(object):
             for errline in stderr.strip().split("\n")[:-1]:
                 logging.error("SYNC Error(Rsync): %s" % errline)
 
-        if gconf.log_rsync_performance:
+        if log_rsync_performance:
             rsync_msg = []
             for line in stdout.split("\n"):
                 if line.startswith("Number of files:") or \
@@ -1468,6 +1524,13 @@ class GLUSTER(AbstractUrl, SlaveLocal, SlaveRemote):
                                               uuid + '.' + gconf.slave_id)
                         ),
                         slave.server)
+                    slave.server.entry_stime = types.MethodType(
+                        lambda _self, path, uuid: (
+                            brickserver.entry_stime(
+                                path,
+                                uuid + '.' + gconf.slave_id)
+                        ),
+                        slave.server)
                     slave.server.set_stime = types.MethodType(
                         lambda _self, path, uuid, mark: (
                             brickserver.set_stime(path,
@@ -1475,6 +1538,14 @@ class GLUSTER(AbstractUrl, SlaveLocal, SlaveRemote):
                                                   mark)
                         ),
                         slave.server)
+                    slave.server.set_entry_stime = types.MethodType(
+                        lambda _self, path, uuid, mark: (
+                            brickserver.set_entry_stime(
+                                path,
+                                uuid + '.' + gconf.slave_id,
+                                mark)
+                        ),
+                        slave.server)
                 (g1, g2, g3) = self.gmaster_instantiate_tuple(slave)
                 g1.master.server = brickserver
                 g2.master.server = brickserver
@@ -1492,7 +1563,12 @@ class GLUSTER(AbstractUrl, SlaveLocal, SlaveRemote):
             changelog_register_failed = False
             (inf, ouf, ra, wa) = gconf.rpc_fd.split(',')
             changelog_agent = RepceClient(int(inf), int(ouf))
-            status = GeorepStatus(gconf.state_file, gconf.local_path)
+            master_name, slave_data = get_master_and_slave_data_from_args(
+                sys.argv)
+            status = GeorepStatus(gconf.state_file, gconf.local_node,
+                                  gconf.local_path,
+                                  gconf.local_node_id,
+                                  master_name, slave_data)
             status.reset_on_worker_start()
             rv = changelog_agent.version()
             if int(rv) != CHANGELOG_AGENT_CLIENT_VERSION:
@@ -1539,7 +1615,7 @@ class GLUSTER(AbstractUrl, SlaveLocal, SlaveRemote):
             except ChangelogHistoryNotAvailable:
                 logging.info('Changelog history not available, using xsync')
                 g1.crawlwrap(oneshot=True, register_time=register_time)
-            except NoPurgeTimeAvailable:
+            except NoStimeAvailable:
                 logging.info('No stime available, using xsync crawl')
                 g1.crawlwrap(oneshot=True, register_time=register_time)
             except ChangelogException as e:
